@@ -1,5 +1,5 @@
 /***************************************************************************
- *   copyright       : (C) 2003-2010 by Pascal Brachet                     *
+ *   copyright       : (C) 2003-2011 by Pascal Brachet                     *
  *   http://www.xm1math.net/texmaker/                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -13,7 +13,7 @@
 #define PDFSCROLLAREA_H
 
 #include <QScrollArea>
-
+#include <QVBoxLayout>
 class PdfScrollArea : public QScrollArea
 {
 Q_OBJECT
@@ -21,15 +21,20 @@ Q_OBJECT
 public:
     PdfScrollArea( QWidget *parent);
     ~PdfScrollArea();
-
+QWidget *scrollAreaWidgetContents;
+QVBoxLayout *verticalLayout;
 signals:
-  void pageup();
-  void pagedown();
+  void doScroll(int);
+  void doRange();
+  void pagezoomOut();
+  void pagezoomIn();
 protected:
   void wheelEvent( QWheelEvent* );
+private slots:
+  void scrollChanged(int value);
+    void rChanged(int min,int max);
 public slots:
-  void scrolltoMin();
-  void scrolltoMax();
+void setVisible(int x, int y, int margin, int maxv);
 };
 
 #endif
